﻿using Communication.Frames;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Communication
{
    public class ResponseParser
    {
        public int ReceivedFrames { get; private set; } = 0;
        public int CrcErrors { get; private set; } = 0;

        public ResponseFrame Parse(byte[] data)
        {
            if (ContainsValidFrame(data))
                return GetFrameFromBytes(data);
            else
                return null;
        }

        public bool ContainsValidFrame(byte[] data)
        {
            if (data[0] != '#')
                return false;

            if (data[ResponseFrame.Length - 1] != '%')
                return false;

            if (data[5] != '$' || data[7] != '$')
                return false;

            ReceivedFrames++;

            ushort frameCrc = (ushort)(data[8] << 8 | data[9]);
            ushort desiredCrc = GetCrc(data);
            
            if (frameCrc != desiredCrc)
            {
                CrcErrors++;
                return false;
            }
                
            return true;
        }

        private ushort GetCrc(byte[] data)
        {
            byte[] frame = (byte[])data.Clone();
            frame[8] = 0;
            frame[9] = 0;
            return CRC16.Get(frame);
        }

        public ResponseFrame GetFrameFromBytes(byte[] data)
        {
            int currentTime = data[1] << 24 | data[2] << 16 | data[3] << 8 | data[4];
            bool state = data[6] != 0;
            return new ResponseFrame(currentTime, state);
        }
    }
}
